@@ -1,10 +1,12 @@
 import json
 from datetime import datetime
+from enum import Enum
 from typing import List
 from unittest import TestCase
 
-from pydictable.core import DictAble, StrField, IntField, ObjectField, ListField, DatetimeField, CustomField, \
-    MultiTypeField
+from pydictable.core import DictAble
+from pydictable.field import IntField, StrField, ListField, ObjectField, DatetimeField, CustomField, MultiTypeField, \
+    EnumField
 
 
 class TestCore(TestCase):
@@ -19,7 +21,7 @@ class TestCore(TestCase):
         address = Address(dict=input_dict)
         self.assertEqual(address.pin_code, 560032)
         self.assertEqual(address.street, 'RT Nagar')
-        self.assertDictEqual(address.to_json(), input_dict)
+        self.assertDictEqual(address.to_dict(), input_dict)
 
     def test_list(self):
         class MessageBox(DictAble):
@@ -29,7 +31,7 @@ class TestCore(TestCase):
         box = MessageBox(dict=input_dict)
         self.assertEqual(len(box.messages), 4)
         self.assertEqual(min(box.messages), 1)
-        self.assertDictEqual(box.to_json(), input_dict)
+        self.assertDictEqual(box.to_dict(), input_dict)
 
         class Message(DictAble):
             message: str = StrField()
@@ -71,7 +73,7 @@ class TestCore(TestCase):
         p = Person(dict=input_dict)
         self.assertEqual(p.address.lat_lng.lat, 12345)
         self.assertEqual(p.address.pin_code, 560032)
-        self.assertDictEqual(p.to_json(), input_dict)
+        self.assertDictEqual(p.to_dict(), input_dict)
 
     def test_default(self):
         class Address(DictAble):
@@ -104,7 +106,7 @@ class TestCore(TestCase):
 
         a = Address()
         a.created_at = datetime(2021, 3, 31)
-        self.assertEqual(a.to_json()['created_at'], 1617129000000)
+        self.assertEqual(a.to_dict()['created_at'], 1617129000000)
 
         a = Address(dict={'created_at': 1617129000000})
         self.assertEqual(a.created_at, datetime(2021, 3, 31))
@@ -126,7 +128,7 @@ class TestCore(TestCase):
             created_at: datetime = DatetimeField()
 
         a = Address(pin_code=560032, created_at=datetime(2021, 3, 31))
-        self.assertEqual(a.to_json()['created_at'], 1617129000000)
+        self.assertEqual(a.to_dict()['created_at'], 1617129000000)
 
     def test_custom_field(self):
         class Car(DictAble):
@@ -145,7 +147,7 @@ class TestCore(TestCase):
                 return CarB(dict=car_dict)
 
         class Garage(DictAble):
-            cars: List[Car] = ListField(CustomField(make_car, lambda c: c.to_json()))
+            cars: List[Car] = ListField(CustomField(make_car, lambda c: c.to_dict()))
 
         g = Garage(dict={
             'cars': [
@@ -153,8 +155,8 @@ class TestCore(TestCase):
                 {'__type': 'CarB', 'b_field': 'b field', 'name': 'I20'}
             ]
         })
-        self.assertEqual(g.to_json()['cars'][0]['name'], 'WagonR')
-        self.assertEqual(g.to_json()['cars'][1]['name'], 'I20')
+        self.assertEqual(g.to_dict()['cars'][0]['name'], 'WagonR')
+        self.assertEqual(g.to_dict()['cars'][1]['name'], 'I20')
 
         class Garage(DictAble):
             cars: List[Car] = ListField(MultiTypeField([CarA, CarB]))
@@ -165,12 +167,77 @@ class TestCore(TestCase):
                 {'__type': 'CarB', 'b_field': 'b field', 'name': 'Mini'}
             ]
         })
-        self.assertEqual(g.to_json()['cars'][0]['name'], 'I10')
-        self.assertEqual(g.to_json()['cars'][1]['name'], 'Mini')
+        self.assertEqual(g.to_dict()['cars'][0]['name'], 'I10')
+        self.assertEqual(g.to_dict()['cars'][1]['name'], 'Mini')
 
         g = Garage(
             cars=[
                 CarA(name='i20', a_field='some value')
             ]
         )
-        self.assertEqual(g.to_json()['cars'][0]['__type'], 'CarA')
+        self.assertEqual(g.to_dict()['cars'][0]['__type'], 'CarA')
+
+    def test_optional(self):
+        class Car(DictAble):
+            name: str = StrField(required=True)
+
+        self.assertRaises(ValueError, lambda: Car(dict={'name': None}))
+        self.assertRaises(ValueError, lambda: Car(dict={}))
+        self.assertRaises(ValueError, lambda: Car(name=None))
+        self.assertRaises(ValueError, lambda: Car())
+        Car(dict={'name': 'Pramod'})
+
+        class Car(DictAble):
+            no_of_gears: int = IntField(required=True)
+        try:
+            Car()
+        except ValueError as e:
+            self.assertTrue('no_of_gears' in str(e))
+
+    def test_custom_attr(self):
+        d = {
+            '_id': 1,
+            'person_name': 'Pramod'
+        }
+
+        class Person(DictAble):
+            id: int = IntField(required=True, key='_id')
+            name: str = StrField(required=True, key='person_name')
+
+        p = Person(dict=d)
+        self.assertEqual(p.name, 'Pramod')
+        self.assertEqual(p.id, 1)
+        d = p.to_dict()
+        self.assertEqual(d['_id'], 1)
+        self.assertEqual(d['person_name'], 'Pramod')
+
+    def test_support_enum(self):
+        class EmployeeType(Enum):
+            ADMIN = 'ADMIN'
+            MANAGER = 'MANAGER'
+            DEV = 2
+
+        d = {
+            'emp_id': 23,
+            'type': 'ADMIN',
+            'roles': ['ADMIN', 'MANAGER', 2]
+        }
+
+        class Employee(DictAble):
+            id: int = IntField(required=True, key='emp_id')
+            type: EmployeeType = EnumField(EmployeeType)
+            roles: List[EmployeeType] = ListField(EnumField(EmployeeType))
+
+        e = Employee(dict=d)
+        self.assertEqual(e.type, EmployeeType.ADMIN)
+        d = e.to_dict()
+        self.assertEqual(d['type'], 'ADMIN')
+        self.assertEqual(e.roles, [EmployeeType.ADMIN, EmployeeType.MANAGER, EmployeeType.DEV])
+        d = e.to_dict()
+        self.assertEqual(d['roles'], ['ADMIN', 'MANAGER', 2])
+        d = {
+            'emp_id': 23,
+            'type': 'ADMIN',
+            'roles': ['ADMIN', 'MANAGER', 4]
+        }
+        self.assertRaises(ValueError, lambda: Employee(dict=d))
